
local blob = {}
blob.__index = blob

local function new(x, y, tileSize)
  return setmetatable({
    x = x or (love.graphics.getWidth() /(tileSize or 10))/2,
    y = y or (love.graphics.getHeight()/(tileSize or 10))/2,
    tileSize = tileSize or 10,
    color = {
      r = 127,
      g = 127,
      b = 127,
      a = 255,
    },
    curMoveTime = 0,
    moveTime = 0.1,
    movableDirections = {
      "r",
      "d",
      "l",
      "u",
    },
    curColBoxX = false,
    curColBoxY = false,
    energy = math.ceil(math.random(100)),
    dead = false,
  }, blob)
end

local function isblob(b)
	return getmetatable(b) == blob
end

function blob:update(dt)
  self.curMoveTime = self.curMoveTime + dt
  if self.curMoveTime > self.moveTime then
    self.curMoveTime = self.curMoveTime - self.moveTime
    
    self:move(math.floor(math.random()*4+1))
  end
end

function blob:draw()
  love.graphics.setColor(self:getColor())
  love.graphics.rectangle("fill", self.x*self.tileSize, 
                                  self.y*self.tileSize, 
                          self.tileSize, self.tileSize)
end

function blob:move(dir)
  assert(self.movableDirections[dir], "move(dir): Invalid direction!")
  
  if dir == 1 then -- right
    self.x = self.x + 1
  elseif dir == 2 then -- down
    self.y = self.y + 1
  elseif dir == 3 then -- left
    self.x = self.x - 1
  elseif dir == 4 then -- up
    self.y = self.y - 1
  end  
end

function blob:collidesWith(otherBlob)
  assert(getmetatable(otherBlob) == blob, "collidesWith: argument   needs to be a Blob!")
  local otherX, otherY = otherBlob:getPosition()
  
  return self.x == otherX and  self.y == otherY
end

function blob:kill()
  self.dead = true
  self:setColor(255, 0, 0, 255) -- This should never be visible
end

function blob:isDead()
  return self.dead
end

function blob:getColBox()
  return self.curColBoxX, self.curColBoxY
end
function blob:setColBox(x, y)
  self.curColBoxX = x
  self.curColBoxY = y
end

function blob:setColor(r, g, b, a)
  self.color.r = r or 127
  self.color.g = g or 127
  self.color.b = b or 127
  self.color.a = a or 255
end
function blob:getColor()
  return self.color.r, self.color.g, self.color.b, self.color.a
end

function blob:getPosition()
  return self.x, self.y
end

function blob:getEnergy()
  return self.energy
end

-- the module
return setmetatable({new = new, isblob = isblob},
	{__call = function(_, ...) return new(...) end})